@@ -6,10 +6,9 @@ use strict;
 use Cwd 'getcwd';
 use File::Basename ();
 use File::Spec     ();
-use English qw( -no_match_vars );
 
 BEGIN {
-    if ( $OLD_PERL_VERSION < 5.006 ) {
+    if ( $] < 5.006 ) {
 
         # can't say 'opendir my $dh, $dirname'
         # need to initialise $dh
@@ -19,22 +18,28 @@ BEGIN {
 
 use Exporter ();
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
-$VERSION   = '2.10_003';
+$VERSION   = '2.10_004';
 $VERSION   = eval $VERSION;
 @ISA       = qw(Exporter);
 @EXPORT    = qw(mkpath rmtree);
 @EXPORT_OK = qw(make_path remove_tree);
 
-my $is_vms   = $OSNAME eq 'VMS';
-my $is_macos = $OSNAME eq 'MacOS';
+BEGIN {
+  for (qw(VMS MacOS MSWin32 os2)) {
+    no strict 'refs';
+    *{"_IS_\U$_"} = $^O eq $_ ? sub () { 1 } : sub () { 0 };
+  }
 
-# These OSes complain if you want to remove a file that you have no
-# write permission to:
-my $force_writeable = grep { $OSNAME eq $_ } qw(amigaos dos epoc MSWin32 MacOS os2);
+  # These OSes complain if you want to remove a file that you have no
+  # write permission to:
+  *_FORCE_WRITABLE = (
+    grep { $^O eq $_ } qw(amigaos dos epoc MSWin32 MacOS os2)
+  ) ? sub () { 1 } : sub () { 0 };
 
-# Unix-like systems need to stat each directory in order to detect
-# race condition. MS-Windows is immune to this particular attack.
-my $need_stat_check = !( $OSNAME eq 'MSWin32' );
+  # Unix-like systems need to stat each directory in order to detect
+  # race condition. MS-Windows is immune to this particular attack.
+  *_NEED_STAT_CHECK = !(_IS_MSWIN32()) ? sub () { 1 } : sub () { 0 };
+}
 
 sub _carp {
     require Carp;
@@ -53,11 +58,11 @@ sub _error {
 
     if ( $arg->{error} ) {
         $object = '' unless defined $object;
-        $message .= ": $ERRNO" if $ERRNO;
+        $message .= ": $!" if $!;
         push @{ ${ $arg->{error} } }, { $object => $message };
     }
     else {
-        _carp( defined($object) ? "$message for $object: $ERRNO" : "$message: $ERRNO" );
+        _carp( defined($object) ? "$message for $object: $!" : "$message: $!" );
     }
 }
 
@@ -139,10 +144,10 @@ sub _mkpath {
     my ( @created );
     foreach my $path ( @{$paths} ) {
         next unless defined($path) and length($path);
-        $path .= '/' if $OSNAME eq 'os2' and $path =~ /^\w:\z/s; # feature of CRT
+        $path .= '/' if _IS_OS2 and $path =~ /^\w:\z/s; # feature of CRT
 
         # Logic wants Unix paths, so go with the flow.
-        if ($is_vms) {
+        if (_IS_VMS) {
             next if $path eq '/';
             $path = VMS::Filespec::unixify($path);
         }
@@ -171,13 +176,13 @@ sub _mkpath {
             }
         }
         else {
-            my $save_bang = $ERRNO;
-            my ( $e, $e1 ) = ( $save_bang, $EXTENDED_OS_ERROR );
+            my $save_bang = $!;
+            my ( $e, $e1 ) = ( $save_bang, $^E );
             $e .= "; $e1" if $e ne $e1;
 
             # allow for another process to have created it meanwhile
             if ( ! -d $path ) {
-                $ERRNO = $save_bang;
+                $! = $save_bang;
                 if ( $arg->{error} ) {
                     push @{ ${ $arg->{error} } }, { $path => $e };
                 }
@@ -253,22 +258,22 @@ sub rmtree {
     for my $p (@$paths) {
 
         # need to fixup case and map \ to / on Windows
-        my $ortho_root = $OSNAME eq 'MSWin32' ? _slash_lc($p) : $p;
+        my $ortho_root = _IS_MSWIN32 ? _slash_lc($p) : $p;
         my $ortho_cwd =
-          $OSNAME eq 'MSWin32' ? _slash_lc( $arg->{cwd} ) : $arg->{cwd};
+          _IS_MSWIN32 ? _slash_lc( $arg->{cwd} ) : $arg->{cwd};
         my $ortho_root_length = length($ortho_root);
-        $ortho_root_length-- if $OSNAME eq 'VMS';   # don't compare '.' with ']'
+        $ortho_root_length-- if _IS_VMS;   # don't compare '.' with ']'
         if ( $ortho_root_length && _is_subdir( $ortho_root, $ortho_cwd ) ) {
-            local $ERRNO = 0;
+            local $! = 0;
             _error( $arg, "cannot remove path when cwd is $arg->{cwd}", $p );
             next;
         }
 
-        if ($is_macos) {
+        if (_IS_MACOS) {
             $p = ":$p" unless $p =~ /:/;
             $p .= ":" unless $p =~ /:\z/;
         }
-        elsif ( $OSNAME eq 'MSWin32' ) {
+        elsif ( _IS_MSWIN32 ) {
             $p =~ s{[/\\]\z}{};
         }
         else {
@@ -313,7 +318,7 @@ sub _rmtree {
 
         if ( -d _ ) {
             $root = VMS::Filespec::vmspath( VMS::Filespec::pathify($root) )
-              if $is_vms;
+              if _IS_VMS;
 
             if ( !chdir($root) ) {
 
@@ -345,7 +350,7 @@ sub _rmtree {
                 next ROOT_DIR;
               };
 
-            if ($need_stat_check) {
+            if (_NEED_STAT_CHECK) {
                 ( $ldev eq $cur_dev and $lino eq $cur_inode )
                   or _croak(
 "directory $canon changed before chdir, expected dev=$ldev ino=$lino, actual dev=$cur_dev ino=$cur_inode, aborting."
@@ -373,7 +378,7 @@ sub _rmtree {
             }
 
             my $d;
-            $d = gensym() if $OLD_PERL_VERSION < 5.006;
+            $d = gensym() if $] < 5.006;
             if ( !opendir $d, $curdir ) {
                 _error( $arg, "cannot opendir", $canon );
                 @files = ();
@@ -389,7 +394,7 @@ sub _rmtree {
                 closedir $d;
             }
 
-            if ($is_vms) {
+            if (_IS_VMS) {
 
                 # Deleting large numbers of files from VMS Files-11
                 # filesystems is faster if done in reverse ASCIIbetical order.
@@ -418,16 +423,16 @@ sub _rmtree {
             # don't leave the client code in an unexpected directory
             chdir( $arg->{cwd} )
               or
-              _croak("cannot chdir to $arg->{cwd} from $canon: $ERRNO, aborting.");
+              _croak("cannot chdir to $arg->{cwd} from $canon: $!, aborting.");
 
             # ensure that a chdir upwards didn't take us somewhere other
             # than we expected (see CVE-2002-0435)
             ( $cur_dev, $cur_inode ) = ( stat $curdir )[ 0, 1 ]
               or _croak(
-                "cannot stat prior working directory $arg->{cwd}: $ERRNO, aborting."
+                "cannot stat prior working directory $arg->{cwd}: $!, aborting."
               );
 
-            if ($need_stat_check) {
+            if (_NEED_STAT_CHECK) {
                 ( $arg->{device} eq $cur_dev and $arg->{inode} eq $cur_inode )
                   or _croak(  "previous directory $arg->{cwd} "
                             . "changed before entering $canon, "
@@ -438,14 +443,14 @@ sub _rmtree {
 
             if ( $arg->{depth} or !$arg->{keep_root} ) {
                 if ( $arg->{safe}
-                    && ( $is_vms
+                    && ( _IS_VMS
                         ? !&VMS::Filespec::candelete($root)
                         : !-w $root ) )
                 {
                     print "skipped $root\n" if $arg->{verbose};
                     next ROOT_DIR;
                 }
-                if ( $force_writeable and !chmod $perm | oct '700', $root ) {
+                if ( _FORCE_WRITABLE and !chmod $perm | oct '700', $root ) {
                     _error( $arg, "cannot make directory writeable", $canon );
                 }
                 print "rmdir $root\n" if $arg->{verbose};
@@ -456,9 +461,9 @@ sub _rmtree {
                 else {
                     _error( $arg, "cannot remove directory", $canon );
                     if (
-                        $force_writeable
+                        _FORCE_WRITABLE
                         && !chmod( $perm,
-                            ( $is_vms ? VMS::Filespec::fileify($root) : $root )
+                            ( _IS_VMS ? VMS::Filespec::fileify($root) : $root )
                         )
                       )
                     {
@@ -475,14 +480,14 @@ sub _rmtree {
         else {
             # not a directory
             $root = VMS::Filespec::vmsify("./$root")
-              if $is_vms
+              if _IS_VMS
               && !File::Spec->file_name_is_absolute($root)
               && ( $root !~ m/(?<!\^)[\]>]+/ );    # not already in VMS syntax
 
             if (
                 $arg->{safe}
                 && (
-                    $is_vms
+                    _IS_VMS
                     ? !&VMS::Filespec::candelete($root)
                     : !( -l $root || -w $root )
                 )
@@ -493,7 +498,7 @@ sub _rmtree {
             }
 
             my $nperm = $perm & oct '7777' | oct '600';
-            if (    $force_writeable
+            if (    _FORCE_WRITABLE
                 and $nperm != $perm
                 and not chmod $nperm, $root )
             {
@@ -508,14 +513,14 @@ sub _rmtree {
                 }
                 else {
                     _error( $arg, "cannot unlink file", $canon );
-                    $force_writeable and chmod( $perm, $root )
+                    _FORCE_WRITABLE and chmod( $perm, $root )
                       or _error( $arg,
                         sprintf( "cannot restore permissions to 0%o", $perm ),
                         $canon );
                     last;
                 }
                 ++$count;
-                last unless $is_vms && lstat $root;
+                last unless _IS_VMS && lstat $root;
             }
         }
     }
